//
//  GameViewController.m
//  Fish
//
//  Created by Daniel Loomb on 22/08/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "GameViewController.h"
#import "FishViewController.h"
#import "LevelSelectViewController.h"
#import "ScoreCalculatorViewController.h"
#import "Sprite.h"
#import "Fish.h"
#import "DustMite.h"
#import "AnimalFur.h"
#import "Pollen.h"
#import "FluVirus.h"
#import "Cold.h"
#import "Smoke.h"
#import "DeathSplat.h"
#import "ScoreImage.h"
#import <stdlib.h>
#import <UIKit/UIKit.h>
#import "PreventerBubble.h"
#import "PreventerWall.h"
#import "PreventerPowerup.h"
#import "RelieverPowerup.h"
#import "LevelController.h"
#import "Sounds.h"

@implementation GameViewController

@synthesize scoreLabel;
@synthesize killTimeLabel;
@synthesize tKindLabel;
@synthesize TimeLabel;
@synthesize levelBeginLabel;

@synthesize continueButton;
@synthesize levelSelectButton;
@synthesize mainMenuButton;
@synthesize pauseButton;
@synthesize backgroundImage;
@synthesize lungOverlayImage;
@synthesize pauseScreenFade;
@synthesize menuBackgroundImage;

@synthesize proceedWin;
@synthesize statusLabel;
@synthesize statusSubLabel;

@synthesize proceedLose;
@synthesize levelSelectLose;
@synthesize mainMenuLose;


Fish *theFish;
NSMutableArray *triggersArray;
PreventerWall *preventerArray;
NSMutableArray *miscUpdateArray;

BOOL usedReliever = FALSE;


NSTimer *gameTimer;
NSTimer *miscTimer;
NSTimer *hideLevelBeginTimer;
BOOL gameLock = FALSE;
int elapsedTimeMilli;
int elapsedTimeSec;
int elapsedTimeMin;

float spawnTimeTotal = 50;
float spawnTimeScaler = 1;
float spawnTimer;
int spawnCounter;
int hideCount;

int score = 0;

Sounds *aSound;

// set the level
-(void)setLevel:(int)cLevel {
	currentLevel = cLevel;
	
	[levelBeginLabel setText:[NSString stringWithFormat:@"Level %i Begin", currentLevel]];
	[levelBeginLabel sizeToFit];
	[levelBeginLabel setCenter:CGPointMake(768/2, levelBeginLabel.center.y)];
	levelBeginLabel.hidden = FALSE;
	hideCount = 50;
	hideLevelBeginTimer = [NSTimer scheduledTimerWithTimeInterval:1/30.0 target:self selector:@selector(HideLevelBeginLabel) userInfo:nil repeats:true];
	
	
}



// set the Fish
-(void)setFish:(Fish *)tFish
{
	theFish = tFish;
	[self.view addSubview:tFish];
	[self.view sendSubviewToBack:theFish];
}

// set the Preventer Array
-(void)setPreventerArray:(PreventerWall *)pArray {
	preventerArray = pArray;
}

// set the Triggers Array
-(void)setTriggersArray:(NSMutableArray *)tArray {
	triggersArray = tArray;
}

-(void)setMiscUpdateArray:(NSMutableArray *)mArray{
	miscUpdateArray = mArray;
}

/*
 * What to do when the pause button is pressed
 */
-(IBAction)goToMainMenu{
	FishViewController *menuView = [[FishViewController alloc] initWithNibName:NULL bundle:NULL];
	
	[self presentModalViewController:menuView animated:NO];
	

}

/*
 * What to do when the pause button is pressed
 */
-(IBAction)pause{
	

	
	//Prevents Screen Locking
	[UIApplication sharedApplication].idleTimerDisabled = NO;

	[self toggleTimer];
	
	[theFish stopAnimating];
	
	//show the menu buttons	
	
	[statusLabel setText:[NSString stringWithFormat:@"Paused"]];
	[statusLabel sizeToFit];
	[statusLabel setCenter:CGPointMake(768/2, statusLabel.center.y)];
	
	pauseScreenFade.hidden = FALSE;
	continueButton.hidden = FALSE;
	levelSelectButton.hidden = FALSE;
	mainMenuButton.hidden = FALSE;
	pauseButton.hidden = TRUE;
	menuBackgroundImage.hidden = FALSE;
	statusLabel.hidden = FALSE;
}


/*
 * What to do when the continue button is pressed
 */
-(IBAction)continueGame{
	

	
	//Prevents Screen Locking
	[UIApplication sharedApplication].idleTimerDisabled = YES;
	
	[self toggleTimer];	
	
	[theFish startAnimating];
	
	//hide the menu buttons
	
	
	
	pauseScreenFade.hidden = TRUE;
	continueButton.hidden = TRUE;
	levelSelectButton.hidden = TRUE;
	mainMenuButton.hidden = TRUE;
	pauseButton.hidden = FALSE;
	menuBackgroundImage.hidden = TRUE;
	statusLabel.hidden = TRUE;
	
}

/*
 *Open the level select menu
 */
-(IBAction)menuItemLevelSelect
{
	LevelSelectViewController * levelView = [[LevelSelectViewController alloc] initWithNibName:NULL bundle:NULL];
	
	[self presentModalViewController:levelView animated:NO];

}


-(IBAction)proceedLoseAction{
	GameViewController *gameView = [[GameViewController alloc] initWithNibName:NULL bundle:NULL];
	[self presentModalViewController:gameView animated:NO];
	[[[LevelController alloc] init:currentLevel:gameView] release];
}

-(void)toggleTimer{
	if (gameLock == FALSE) {
		
		if(gameTimer == nil){//if the timer is currently empty then set it
			gameTimer = [NSTimer scheduledTimerWithTimeInterval:1/30.0 target:self selector:@selector(updateGame) userInfo:nil repeats:true];
			
			for (int i = ([triggersArray count]-1); i >= 0; i--) {
				Sprite *updateTrigger = [triggersArray objectAtIndex:i];		
				[updateTrigger startAnimating];
			}
			
			
			for (int i = ([miscUpdateArray count]-1); i >= 0; i--) {
				Sprite *updateTrigger = [miscUpdateArray objectAtIndex:i];		
				[updateTrigger startAnimating];
			}
			[aSound BGMResume];
		}
		else{//else stop the timer 
			[gameTimer invalidate];
			gameTimer = nil;
			
			for (int i = ([triggersArray count]-1); i >= 0; i--) {
				Sprite *updateTrigger = [triggersArray objectAtIndex:i];		
				[updateTrigger stopAnimating];
			}
			
			
			for (int i = ([miscUpdateArray count]-1); i >= 0; i--) {
				Sprite *updateTrigger = [miscUpdateArray objectAtIndex:i];		
				[updateTrigger stopAnimating];
			}
			[aSound BGMPause];
		}
		
	}
}

-(void) lockTimer{
	gameLock = !gameLock;
}



// The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
		
		spawnTimeScaler = 1;
		spawnTimer = spawnTimeTotal;
		spawnCounter = 0;
		
		aSound = [[Sounds alloc] initShared];
		[aSound BGMStart:arc4random() % 3];
		
		score = 0;
		elapsedTimeMilli = 0;
		elapsedTimeSec = 0;
		elapsedTimeMin = 0;
		
		
		//Prevents Screen Locking
		[UIApplication sharedApplication].idleTimerDisabled = YES;
		
		//instatiate timer
		gameTimer = [NSTimer scheduledTimerWithTimeInterval:1/30.0 target:self selector:@selector(updateGame) userInfo:nil repeats:true];
		gameLock = FALSE;
		
		
		//A sneaking way around reseting the reliever -- 
		//(WARNING - I changed the 4th arg to be preventerArray to stop a code warning, should really be triggerArray)
		RelieverPowerup *aReliever = [[RelieverPowerup alloc] init:2000 :2000 : theFish : preventerArray : miscUpdateArray];		
		[aReliever resetReliever];
		[aReliever release];
		//---------------------------
		
		
		usedReliever = FALSE;
		
		[self.view sendSubviewToBack:backgroundImage];
		
		

		
		//[NSTimer scheduledTimerWithTimeInterval:10 target:self selector:@selector(forceWin) userInfo:nil repeats:NO]; // calls the win event
    }
    return self;
}

-(void) HideLevelBeginLabel{
		if (hideCount == 0) {
			levelBeginLabel.hidden = TRUE;
			[hideLevelBeginTimer invalidate];
			hideLevelBeginTimer = nil;
		}
		else {
			hideCount--;
		}

}

- (BOOL) checkIfLost {
	for (int i = 0; i < [preventerArray count]-1; i++) {
		NSMutableArray *tempArray = [preventerArray objectAtIndex:i];
		
		
		if ([tempArray count] == 0) {//if this bubble was the last in its array we have a breach so do...
			[self toggleTimer];
			[self lockTimer];
			
			[statusLabel setText:[NSString stringWithFormat:@"You Lost"]];
			[statusLabel sizeToFit];
			[statusLabel setCenter:CGPointMake(768/2, statusLabel.center.y)];
			
			[statusSubLabel setText:[NSString stringWithFormat:@"Enemies breached the preventer lining"]];
			[statusSubLabel sizeToFit];
			[statusSubLabel setCenter:CGPointMake(768/2, statusSubLabel.center.y)];
			
			//sort layout
			[self.view bringSubviewToFront:menuBackgroundImage];
			[self.view bringSubviewToFront:levelSelectLose];
			[self.view bringSubviewToFront:mainMenuLose];
			[self.view bringSubviewToFront:proceedLose];
			[self.view bringSubviewToFront:statusLabel];
			[self.view bringSubviewToFront:statusSubLabel];
			
			pauseButton.hidden = TRUE;
			levelSelectLose.hidden = FALSE;
			menuBackgroundImage.hidden = FALSE;
			mainMenuLose.hidden = FALSE;
			proceedLose.hidden = FALSE;
			statusLabel.hidden = FALSE;
			statusSubLabel.hidden = FALSE;
			

			//[fishMusic stop];
			[theFish stopAnimating];

			
			return TRUE;
		}
	}
	return FALSE;
	
}

-(void)forceWin {
	[self toggleTimer];
	[self lockTimer];
	
	/*[statusLabel setText:[NSString stringWithFormat:@"You Won"]];
	 
	 [statusLabel sizeToFit];
	 [statusLabel setCenter:CGPointMake(768/2, statusLabel.center.y)];
	 
	 //sort layout
	 [self.view bringSubviewToFront:menuBackgroundImage];
	 [self.view bringSubviewToFront:proceedWin];
	 [self.view bringSubviewToFront:statusLabel];
	 
	 menuBackgroundImage.hidden = FALSE;
	 pauseButton.hidden = TRUE;
	 proceedWin.hidden = FALSE;
	 statusLabel.hidden = FALSE;*/
	
	[theFish stopAnimating];

	miscTimer = [NSTimer scheduledTimerWithTimeInterval:1/30.0 target:self selector:@selector(updateMiscTriggersByThemselves) userInfo:nil repeats:true];
}

- (BOOL) checkIfWon {
	//Temp Win Condition
	//printf("HIT! numMites = %d\n", [aTriggerArray count]);
	if ([triggersArray count]  < 1) {
		[self forceWin];
		return TRUE;
	}
	return FALSE;
	
}

-(void)updateMiscTriggersByThemselves{
	//update misc objects
	//if ([miscUpdateArray count] != 0) {
	BOOL scoreImageCheck = FALSE;
	for (int j = [miscUpdateArray count] -1; j >= 0 ; j--) {
		//printf("j = %i \n", j);
		if ([[miscUpdateArray objectAtIndex:j] isKindOfClass:[RelieverPowerup class]]) {
			usedReliever = [[miscUpdateArray objectAtIndex:j] checkIfRelieverWasUsed];
			[[miscUpdateArray objectAtIndex:j] countdown];
		}
		else {
			if ([[miscUpdateArray objectAtIndex:j] isKindOfClass:[ScoreImage class]] || [[miscUpdateArray objectAtIndex:j] isKindOfClass:[DeathSplat class]]) {
				scoreImageCheck = TRUE;
			}
			[[miscUpdateArray objectAtIndex:j] countdown];
		}
	}
	if (scoreImageCheck == FALSE) {
		[miscTimer invalidate];
		printf("Current Level = %i\n", currentLevel);
		ScoreCalculatorViewController *scoreView = [[ScoreCalculatorViewController alloc] initSelf: elapsedTimeMin : elapsedTimeSec : score : usedReliever : currentLevel :preventerArray];//initWithNibName:NULL bundle:NULL : elapsedTimeMin : elapsedTimeSec : score : TRUE];
		
		[self presentModalViewController:scoreView animated:NO];
	}
}

-(void)updateGame{
	
	if (spawnTimer < 0) {
		spawnTimeScaler = spawnTimeScaler - 0.05;
		
		if (spawnTimeScaler < 0.2) {
			spawnTimeScaler = 0.2;
		}
		spawnTimer = spawnTimeTotal * spawnTimeScaler;
				
		if (spawnCounter < [triggersArray count]-1) {
			spawnCounter++;
		}
	}
	spawnTimer--;
	
	//update the triggers
	int i = spawnCounter;
	if (i > [triggersArray count]-1) {
		i = [triggersArray count]-1;
	}
	for (i; i >= 0; i--) {
		Sprite *updateTrigger = [triggersArray objectAtIndex:i];		
		[updateTrigger updateMe];
		score = [updateTrigger getScore];
	}
	
	[theFish updateFish];
	
	//update misc objects
	//if ([miscUpdateArray count] != 0) {
		for (int j = [miscUpdateArray count] -1; j >= 0 ; j--) {
			//printf("j = %i \n", j);
			if ([[miscUpdateArray objectAtIndex:j] isKindOfClass:[RelieverPowerup class]]) {
				usedReliever = [[miscUpdateArray objectAtIndex:j] checkIfRelieverWasUsed];
				[[miscUpdateArray objectAtIndex:j] countdown];
			}
			else {
				[[miscUpdateArray objectAtIndex:j] countdown];
			}
			
		//}
		
	}
	
	
	//Give player feed back of combo multipliers
	if([[triggersArray lastObject] decrementKillTimer] == TRUE){
		killTimeLabel.hidden = FALSE;
	}
	else {
		killTimeLabel.hidden = TRUE;
		tKindLabel.hidden = TRUE;
	}
	
	
	//Calculate Time then format it
	elapsedTimeMilli++;	
	if (elapsedTimeMilli > 29) {
		elapsedTimeSec++;
		elapsedTimeMilli = 0;
		if (elapsedTimeSec > 59) {
			elapsedTimeMin++;
			elapsedTimeSec = 0;
		}
	}	
	
	if (elapsedTimeSec < 10) {
		[TimeLabel setText:[NSString stringWithFormat:@"Time: %i:0%i", elapsedTimeMin, elapsedTimeSec]];
	}
	else {
		[TimeLabel setText:[NSString stringWithFormat:@"Time: %i:%i", elapsedTimeMin, elapsedTimeSec]];
	}
	
	
	
	
	//calculate the score and update the score label			
	NSString *theScore = [NSString stringWithFormat:@"Score : %i",score];			  
	[scoreLabel setText:theScore];
//	[scoreLabel setFont:[UIFont fontWithName:@"Suplexmentary Comic NC" size:36]];
	
	
	//Make View Layout Correct
	[self.view sendSubviewToBack:lungOverlayImage];
	[self.view sendSubviewToBack:backgroundImage];
	[self.view bringSubviewToFront:theFish];
	[self.view bringSubviewToFront:levelBeginLabel];
	[self.view bringSubviewToFront:pauseScreenFade];
	[self.view bringSubviewToFront:menuBackgroundImage];
	[self.view bringSubviewToFront:statusLabel];
	[self.view bringSubviewToFront:mainMenuButton];
	[self.view bringSubviewToFront:continueButton];
	[self.view bringSubviewToFront:levelSelectButton];
	
	
	
	//Check if Game is over
	if ([self checkIfWon]); 
	if (usedReliever != TRUE && [self checkIfLost]);
	
	
	
	
	
}



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	
	//set all the Fonts to our games font
	[continueButton.titleLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[proceedWin.titleLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[proceedLose.titleLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[levelSelectLose.titleLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[mainMenuLose.titleLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[levelSelectButton.titleLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[mainMenuButton.titleLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[scoreLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[TimeLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	tKindLabel.hidden = TRUE;
	killTimeLabel.hidden = TRUE;
	[tKindLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[killTimeLabel setFont:[UIFont fontWithName:@"AppleCasual" size:36]];
	[statusLabel setFont:[UIFont fontWithName:@"AppleCasual" size:48]];
	[statusSubLabel setFont:[UIFont fontWithName:@"AppleCasual" size:24]];
	[levelBeginLabel setFont:[UIFont fontWithName:@"AppleCasual" size:48]];
	
		
	/*
	 ............  ...................   . .     . ............................................  . ...... $M. . .................................
	 .   .  ..  ................... 7MMM~ .:MMMMMMI..,,,,,, .... .   .... .....  ..........  .?NMMMMMM+M~..=M. 8MMMMMMMMO ........................
	 ...........................DMM .. ..  MM     . O88MO88 .... MMZ...,M~.....=M. .........MM.  M.....M~  =M. ,M.................................
	 .  ........................MM+.. ...  MM..........M ........M MM...M~.....=M. ..............M.....MM887M.  MMM ..............................
	 ............   .  .   .  .....OMMMMM. MMOOOO=.....M .......=MMMMMD.M~.....=M. ..............M.....M~..=M. .M.................................
	 ..............  .. ..  .. .....  ,$M. OM~.  .  ...M ...... OM ..M..M:   ..=M.... ...........M ....M~.  O.. M.................................
	 .  ... .  .    .. ..  .. ....7IIII. , I+7$7I ....: ...... ,~ ..M..MMMMMM  MMMMMMN .........M.... ~....... MMMMM.............................
	 ........ .. .. ...... ... ..............................~~~~ZOO8MMMMMMMMMMMMMMMMMZ: .......O..............     .............................
	 ....... ....... ...... ...... ...... ...... ...... ...  ............   .  .   ....M ..  .   .  .   .  .                                     
	 .......     ..  .. ..  .. ... .. ... .. ... ...... .......................................................... ...... ...... ...... ...... ..
	 . ..... .  .  . ........ .... ...... ...... ...... ...  ..  .  ..  .  ..  .  ..  .  ..  .  ..  .  ..  .   .  .   .  . ........ .  .   .  .  
	 .......  .. ..  .. ..  .....  .. ... .. ... .. ... ..............................................  ....... .. ... .. ....=+.... .. ... .. ..
	 ....... .  ....... .......+==....... ...... .MMMMMM....=MMMM~  .Z...  8 .,:::::,.. .?MMMD   .  .M=MM  .   .  .   .. .:++=+,... .  .   .  .  
	 ....... ....... ......  ..~++++~.... ......  M.... ..DMI .  MM ?MN..  M ?ZZMMZZ+ ~MM: . M.  .  . MMD  .         ..,++++++....               
	 .......     ..  .. ..  ....++++++:.. .. ... .MDNMM..?M  .....M M$MM ..M....MM ...OMN. ..M, ..... . =............++++++++ .. ...... ...... ..
	 . ..... .  .  . ...... ....+++=+++:. ......  M.  . .MI  ..  ND M= MM.NN   .MM .  . ~8MMMM7 ..  .  :.  .   ...+=+=+++++=.....   .  .   .  .  
	 ............  ..  .....  ..,+++++++=........ M .... $MMM7ZMM7..M.  DMM~.  .NM..  MMD..,?MO ...,=~,8.  ....=++++++++++,......................
	 . ..........................+++++++++=.......O......8++?,,  ...M ..=+M..  ..  .  . :?=~....==++~..M....:++++++++++++........................
	 ....... .  ... .   .. .  ...~++++++++++~.............+++..........,+==~....... ...... ..:+++=+....M..+=++++++++++++.........................
	 .........  .....  .....  ....++++++++++++=...........++++...... ..+++++...... ........:++++++.....N+++++++++++++++..........................
	 .............................=++++++++++++++,.......:+++++,......,+++++.............,+++++++,...++$++++++++++++++...........................
	 ............  ..  .....  .....=+=+++++++++++++~.....,++++++=.....=+++++,  ........,+++++++==.+++++?++++++++++=++............................
	 . .............................+++++++++++++++++:...,++++++++. ..+++++==  ... ...+++++++++++=++++I++++++++++++=.............................
	 ....... .  ... .   .. .  ......+=+++++++++++++++++..,+++++++++~..+++++++......:++++++++++++++++++N++++++++++++..............................
	 .........  .....  .....  .......++++++++++++++++++=~,++++++++++===++++++....++++=++++++++++++++++M+++++++++++...............................
	 ...........  ..  .....  .......=++++++++++++++++++++++++++++=++++++=+++:=+++++++++++=+++++++++++M+++++++++=................................
	 .................................=+++++++++++++++++++++++++OM8=:::::::MM+++++++++=++DMMM??+++++++M+++++++++.................................
	 ....... .  .....   .. .  ........~++++++++++++++++++++++MO::,::,::::::,~MO++++++++M=.....M++++++??++++++++..................................
	 .........  .,++=.......  .........+++++++++++++++++++++++++NO::::::::::::M==NNNNMM,. .., .M=++++N++++++++...................................
	 ..............=++++=...............+++++++++++++++++++++++++=NN::::DMM=:.~.......M.... . .M+++++M+++++++....................................
	 ............  ..++++++=............~+++++++++++++++++++++++++=?ZM,......  ..... .MM...   N?=++++M=+++++.....................................
	 ..................+++++++=..........+++++++++++++++++++++++++DM. ..... .  .    .. MM8 .,N.:I++++D+++++~.....................................
	 ....... .  ... ....=++++++++=........+++++++++++++++++++?M~.+M,. .......OD+Z. ..+Z  ,~.  ..=I+++I=++++......................................
	 ............... .....+++=++++++=.....~+++++++++++++++++I~.... Z..  ..MM. $?MOMMMMM$.........D++7?++++.......................................
	 ......MNO.,... .  ....++++++++++++++,.=+++++++++++++++=M O.....N .MM...MMMMMMMMMMMMD.........M+D+=++:.......................................
	 .....MM::MZ....... ....++++++++++++++++++++++++++++++++O,.... ~:M .?8MMMMMMMMMMMMMMM.........=IN++++........................................
	 :...$D::::~M$.. ........~+++++++++++++++++++++++++++++++88 . M+M .MMMMMMMMMMMMMMMMMMM.........DM+++..................................:++==..
	 M:N~::~::::::M..  .   .  :+++++++++++++++++++++++++++++++++M. D.:MMMMMMMMMMMMMMMMMMMM.........MM+++............................ ,++++++~....
	 +N::::::::::::N+...... ....=+++++++++++++++++++++++++++++++M.  MNMMMMMMMMMMMMMMMMMMMM.........MD++.....:=+++++=,............:++++++++:......
	 ~:::::::::::::~:M..   .......+++++++++++++++++++++++++++++++=..  MMMMMMMMMMMMMMMMMMMM=.......ZMI++=+==+++++++:...........++++++++=+,........
	 =~:::::::::::::::O....:++++++++++++++++++++++++++++++++++++7M......MMMMMMMMMMMMMMMMMMM.......NZ?+++++++++++=.........,+++++++++++:..........
	 Z?:::::::::::::::I+ . ...++=++++++++++++++++++++++++++++++++OO......7MMMMMMMMMMMMMMMMM.......M7N++++++++++..... .,+++++++++++++.............
	 .M~:::::::::::::::M  . ...+++++++++++++++++++++++++++++++++++M~.......MMMMMMMMMMMMMMN=.......Z7N+++++++++.....:++++++++++++++...............
	 ..M~:::::::::::::::,. .  ...++++++++++++++++++++++++++++++++++M.......DMMMMMMMMMMMMMMZ..... MI77O++++++=...+=+++++++++++=+~.................
	 ...M~::::::::7:::~:,.........=+++++++++++++++++++++++++++++++++M......:MMMMMMMMMMMM8.+.....$$I77M?+++=~,=+++++++++++++++~...................
	 ....M?::::::::::~:O  . .. .....+=++++++++++++++++++++++++++++=N?M.....,8MMMMMMMMMM, O......NI77I$D++++++++++++++++++++:.....................
	 .....MM~::::~M:M8ON....  .......:=+=+++++++++++++++++++++++8M=++O?.....D~ .MNND+ .+M......MI7777IM++++++++++++++++++:.......................
	 ....... ......DOOOOM.. ...........++++++++++++++++++++++IM?++++++M$....  .ZMMMMMOO~ .... M$I7777I7?+++++++++++++++:.........................
	 ...............~NOOO8+...:=+++++=+++++++++++++++++++++MN+=++++++++M?.......   ..........+877777777N++++++++++++++...........................
	 .................MOOOOM+++++++++=++++++++++++++++++8M+=++++++++++++IM.................MM$I7I777777M+++++++++++++............................
	 ...............,++MOOOOM+++++++++++++++++++++++++M7+++++++++++++++++M7MN,...     .?MN7I777777777777?++++++++++..............................
	 ..................~$MOOON7++++++++++++++++++++NM+++++++++++++++++++++M7I777$ZZZZZ77777777777777777IM++++++++:...............................
	 ......++++++~, ......MOOOO8++++++++++++++++IM++++++++++++++++++++++++8I77777II77777777777777777777ID+++++++++++++++++++++++++++=:,,.........
	 ......,+++++++++++=:..N8OOOM+++++++++++++N8+++++++++++++++++++++++++++M7777777777777777777777777777IO+++++++++++++++++++++++++++++++++++~...
	 .......,++++++++=++++++7MOOOM+++++++++8M+=+++++++++++++++++++++++++++++D777777777777777777777777777IM++++++++++++++++++++++++++++++++++++++,
	 .........++++++++++++++++MOOOO8++++=MI+++++++++++++++++++++++++++++++++D77I7777777777777777777777777Z7+++++++++++++++++++++++++++++++++++++=
	 .........,++++++++++++++++M8OO8M+N8+++++++++++++++++++++++++++++++++++++D7I7777777777777777777777777IM=++++++++++++++++++++++++++++++~,.....
	 ..........=+++++++++++++++++NO8MM=++++++++++++++++++++++++++++++++++++++D777777777777777777777777777ID++++++++++++++++++++++++++=...........
	 ...........+++++++++++++++++NMOOO8M+++++++++++++++++++++++++++++++++++++=MI777777777777777777777777777?++++++++++++++++=+++=,...............
	 ............++++++++++++++++++M8OOOM+++++++++++++++++++++++++++++++++++++MI7777777777777777777777777IID++++++++++++++=+~....................
	 .............++++++++++++++++++7NOOOM++++++++++++++++++++++++++++++++++++M77777777777777777777777777IID++++++++++++=........................
	 ..............++++++++++++++++=+?M8OON8++++++++++++++++++++++++++++++++++MI7777777777777777777777777IID+++++++++,...........................
	 ...............+++++++++++++++++++MOOOOM+++++++++++++++++++++++++++++++++N77777777777777777777777777IID++++++...............................
	 ................=++++++++++++++++++INOOOM=++++++++++++++++++++++++++++++7$77777777777777777777777777IID++++++...............................
	 .............. ..,+++++++++++++++++++NOOO88+++++++++++++++++++++++++++++MI77777777777777777777777777IID++++++++:............................
	 ............... ...+++++++++++++++++++NOOOOM++++++++++++++++++++++++++++M777777777777777777777777777IID++++++++++...........................
	 .............. ....,+++++++++++++++++++N8OOOM+=+++++++++++++++++++++++++ZI77777777777777777777777777IID+++++++++++..........................
	 */
	
	[super viewDidLoad];
}






- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
	[super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc. that aren't in use.
}


- (void)viewDidUnload {
	[super viewDidUnload];
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
	[super dealloc];
}


@end
